/*!
 * @file  callbacks.cpp
 * @brief gtk/gdkのコールバック関数関連
 * @author Tsuyoshi SUENAGA
 * @date $Date$
 *
 * $Id$
 */
#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include <gtk/gtk.h>
#include <X11/Xlib.h>

#include "callbacks.h"
#include "interface.h"
#include "support.h"

/// リミッタマクロ thには正の数を指定
#define LIMIT(x,th) ((x>th) ? th : ((x<-th) ? -th : x))

/// 矢印描画のためのパラメータ（羽の長さ）
#define ARROW_L (10)
/// 矢印描画のためのパラメータ（羽の開き量）
#define ARROW_W (5)

/// クリック状態
static int press_flag = 0;
/// X座標の値
static double horizon;
/// Y座標の値
static double vertical;
/// 正負符号
static double sign[2]={1,1};

/** GTKの初期化関連
 */
void
initialize_for_gtk(void)
{
  XInitThreads();
  g_thread_init(NULL);
  gdk_threads_init();

  gtk_set_locale ();
  //gtk_init (&argc, &argv);
  gtk_init (NULL, NULL);
  gdk_rgb_init();
}

/** GTKのメインスレッド
 */
void *
gtk_main_thread(void)
{
  gtk_main ();
  return NULL;
}

/** X座標の値を得る関数
 * @return ドラッグ中のX座標の値を返す
 */
double
get_horizon(void)
{
  return horizon;
}

/** Y座標の値を得る関数
 * @return ドラッグ中のY座標の値を返す
 */
double
get_vertical(void)
{
  return vertical;
}

/** X,Y軸の向き変更を行う関数
 * @param x X座標の軸向き（正：右向き，負：左向き，ゼロ：軸なし）
 * @param y Y座標の軸向き（正：上向き，負：下向き，ゼロ：軸なし）
 */
void
set_sign(double x, double y)
{
  sign[0] = x;
  sign[1] = y;
}

/** 描画実体
 * @param widget drawingareaウィジェット
 */
void
draw_background(GtkWidget *widget)
{
  cairo_t *cr;
  double x0 = widget->allocation.x; // ウィンドウの左上座標x
  double y0 = widget->allocation.y; // ウィンドウの左上座標y
  double width = widget->allocation.width;   // ウィンドウの幅
  double height = widget->allocation.height; // ウィンドウの高さ
  double xc = x0 + width / 2.0;  // ウィンドウの中心座標
  double yc = y0 + height / 2.0; // ウィンドウの中心座標

  // cairoオブジェクトの作成
  cr = gdk_cairo_create(widget->window);

  // 線の色を設定
  if(press_flag == 1)
    cairo_set_source_rgb(cr, 1, 0, 0); // 赤色
  else
    cairo_set_source_rgb(cr, 0, 0, 1); // 青色
  // X軸線
  if(sign[0]>0){
    cairo_move_to(cr, x0, yc);
    cairo_line_to(cr, x0+width, yc);
    cairo_line_to(cr, x0+width-ARROW_L, yc-ARROW_W);
    cairo_move_to(cr, x0+width, yc);
    cairo_line_to(cr, x0+width-ARROW_L, yc+ARROW_W);
  }else if(sign[0]<0){
    cairo_move_to(cr, x0+width, yc);
    cairo_line_to(cr, x0, yc);
    cairo_line_to(cr, x0+ARROW_L, yc-ARROW_W);
    cairo_move_to(cr, x0, yc);
    cairo_line_to(cr, x0+ARROW_L, yc+ARROW_W);
  }else{
    ; // do nothing
  }
  // Y軸線
  if(sign[1]>0){
    cairo_move_to(cr, xc, y0+height);
    cairo_line_to(cr, xc, y0);
    cairo_line_to(cr, xc+ARROW_W, y0+ARROW_L);
    cairo_move_to(cr, xc, y0);
    cairo_line_to(cr, xc-ARROW_W, y0+ARROW_L);
  }else if(sign[1]<0){
    cairo_move_to(cr, xc, y0);
    cairo_line_to(cr, xc, y0+height);
    cairo_line_to(cr, xc+ARROW_W, y0+height-ARROW_L);
    cairo_move_to(cr, xc, y0+height);
    cairo_line_to(cr, xc-ARROW_W, y0+height-ARROW_L);
  }else{
    ; // do nothing
  }    
  // 全描画
  cairo_stroke(cr);

  // cairoオブジェクトの削除
  cairo_destroy(cr);
}

void
on_window1_destroy                     (GtkObject       *object,
                                        gpointer         user_data)
{
  //gtk_main_quit();
}


gboolean
on_drawingarea1_button_press_event     (GtkWidget       *widget,
                                        GdkEventButton  *event,
                                        gpointer         user_data)
{
  press_flag = 1;
  return FALSE;
}


gboolean
on_drawingarea1_button_release_event   (GtkWidget       *widget,
                                        GdkEventButton  *event,
                                        gpointer         user_data)
{
  press_flag = 0;
  horizon  = 0.0;
  vertical = 0.0;
  fprintf(stderr, "(%.2f, %.2f)\n", horizon, vertical);
  return FALSE;
}


gboolean
on_drawingarea1_motion_notify_event    (GtkWidget       *widget,
                                        GdkEventMotion  *event,
                                        gpointer         user_data)
{
  int mouse_x, mouse_y;// マウス座標
  double x0 = widget->allocation.x; // ウィンドウの左上座標x
  double y0 = widget->allocation.y; // ウィンドウの左上座標y
  double width = widget->allocation.width;   // ウィンドウの幅
  double height = widget->allocation.height; // ウィンドウの高さ
  double xc = x0 + width / 2.0;  // ウィンドウの中心座標
  double yc = y0 + height / 2.0; // ウィンドウの中心座標
  double gain_x = (width  - 1.0)/2.0;
  double gain_y = (height - 1.0)/2.0;
  double rate_x, rate_y; // 比率 -1〜1

  // 描画する
  draw_background(widget);
  
  // マウスポインタ座標を受け取る
  mouse_x = (int)event->x;
  mouse_y = (int)event->y;

  // ウィンドウ中心を原点としてマウス位置を-1〜1で表す
  rate_x = (mouse_x - xc) / gain_x;
  rate_y = (yc - mouse_y) / gain_y;
  rate_x = LIMIT(rate_x, 1.0);
  rate_y = LIMIT(rate_y, 1.0);

  if(press_flag==1){
    horizon  = rate_x;
    vertical = rate_y;
    //fprintf(stderr, "(%.2f, %.2f)\n", horizon, vertical);
  }
  
  return FALSE;
}


gboolean
on_drawingarea1_expose_event           (GtkWidget       *widget,
                                        GdkEventExpose  *event,
                                        gpointer         user_data)
{
  /// 背景の座標軸を描画
  draw_background(widget);
  return FALSE;
}

